@@ -1,37 +1,18 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "wizard.h"
 
 #include "algorithm.h"
+#include "guiutils.h"
 #include "hostosinfo.h"
+#include "icon.h"
 #include "qtcassert.h"
+#include "theme/theme.h"
+#include "utilstr.h"
 #include "wizardpage.h"
 
-#include <utils/theme/theme.h>
-
+#include <QDebug>
 #include <QDialog>
 #include <QDialogButtonBox>
 #include <QHash>
@@ -40,11 +21,13 @@
 #include <QLabel>
 #include <QMap>
 #include <QScrollArea>
-#include <QVBoxLayout>
 #include <QVariant>
+#include <QVBoxLayout>
 
 
-/*! \class Utils::Wizard
+/*!
+  \class Utils::Wizard
+  \inmodule QtCreator
 
   \brief The Wizard class implements a wizard with a progress bar on the left.
 
@@ -63,7 +46,9 @@ public:
         m_indicatorPixmap(indicatorPixmap)
     {
         m_indicatorLabel = new QLabel(this);
-        m_indicatorLabel->setFixedSize(m_indicatorPixmap.size());
+        const QSizeF indicatorSize = m_indicatorPixmap.deviceIndependentSize();
+        m_indicatorLabel->setFixedSize(
+            {qCeil(indicatorSize.width()), qCeil(indicatorSize.height())});
         m_titleLabel = new QLabel(title, this);
         auto l = new QHBoxLayout(this);
         l->setContentsMargins(0, 0, 0, 0);
@@ -115,8 +100,7 @@ private:
     QVBoxLayout *m_mainLayout;
     QVBoxLayout *m_itemWidgetLayout;
     WizardProgress *m_wizardProgress;
-    QMap<WizardProgressItem *, ProgressItemWidget *> m_itemToItemWidget;
-    QMap<ProgressItemWidget *, WizardProgressItem *> m_itemWidgetToItem;
+    QHash<WizardProgressItem *, ProgressItemWidget *> m_itemToItemWidget;
     QList<WizardProgressItem *> m_visibleItems;
     ProgressItemWidget *m_dotsItemWidget;
     int m_disableUpdatesCount;
@@ -129,7 +113,7 @@ LinearProgressWidget::LinearProgressWidget(WizardProgress *progress, QWidget *pa
     m_dotsItemWidget(nullptr),
     m_disableUpdatesCount(0)
 {
-    m_indicatorPixmap = QIcon::fromTheme(QLatin1String("go-next"), QIcon(QLatin1String(":/utils/images/arrow.png"))).pixmap(16);
+    m_indicatorPixmap = Icon::fromTheme("go-next").pixmap(16);
     m_wizardProgress = progress;
     m_mainLayout = new QVBoxLayout(this);
     m_itemWidgetLayout = new QVBoxLayout();
@@ -137,7 +121,7 @@ LinearProgressWidget::LinearProgressWidget(WizardProgress *progress, QWidget *pa
     m_mainLayout->addLayout(m_itemWidgetLayout);
     m_mainLayout->addSpacerItem(spacer);
 
-    m_dotsItemWidget = new ProgressItemWidget(m_indicatorPixmap, tr("..."), this);
+    m_dotsItemWidget = new ProgressItemWidget(m_indicatorPixmap, Tr::tr("..."), this);
     m_dotsItemWidget->setVisible(false);
     m_dotsItemWidget->setEnabled(false);
 
@@ -156,9 +140,9 @@ LinearProgressWidget::LinearProgressWidget(WizardProgress *progress, QWidget *pa
     connect(m_wizardProgress, &WizardProgress::currentItemChanged,
             this, &LinearProgressWidget::slotCurrentItemChanged);
 
-    QList<WizardProgressItem *> items = m_wizardProgress->items();
-    for (int i = 0; i < items.count(); i++)
-        slotItemAdded(items.at(i));
+    const QSet<WizardProgressItem *> items = m_wizardProgress->items();
+    for (WizardProgressItem *item : items)
+        slotItemAdded(item);
     recreateLayout();
 
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
@@ -170,21 +154,17 @@ void LinearProgressWidget::slotItemAdded(WizardProgressItem *item)
     itemWidget->setVisible(false);
     itemWidget->setWordWrap(item->titleWordWrap());
     m_itemToItemWidget.insert(item, itemWidget);
-    m_itemWidgetToItem.insert(itemWidget, item);
 }
 
 void LinearProgressWidget::slotItemRemoved(WizardProgressItem *item)
 {
-    ProgressItemWidget *itemWidget = m_itemToItemWidget.value(item);
-    if (!itemWidget)
+    const auto it = m_itemToItemWidget.constFind(item);
+    if (it == m_itemToItemWidget.constEnd())
         return;
 
-    m_itemWidgetToItem.remove(itemWidget);
-    m_itemToItemWidget.remove(item);
-
+    delete *it;
+    m_itemToItemWidget.erase(it);
     recreateLayout();
-
-    delete itemWidget;
 }
 
 void LinearProgressWidget::slotItemChanged(WizardProgressItem *item)
@@ -230,12 +210,9 @@ void LinearProgressWidget::recreateLayout()
 {
     disableUpdates();
 
-    QMap<WizardProgressItem *, ProgressItemWidget *>::ConstIterator it = m_itemToItemWidget.constBegin();
-    QMap<WizardProgressItem *, ProgressItemWidget *>::ConstIterator itEnd = m_itemToItemWidget.constEnd();
-    while (it != itEnd) {
-        it.value()->setVisible(false);
-        ++it;
-    }
+    for (ProgressItemWidget *itemWidget : std::as_const(m_itemToItemWidget))
+        itemWidget->setVisible(false);
+
     m_dotsItemWidget->setVisible(false);
 
     for (int i = m_itemWidgetLayout->count() - 1; i >= 0; --i) {
@@ -244,8 +221,8 @@ void LinearProgressWidget::recreateLayout()
     }
 
     m_visibleItems = m_wizardProgress->directlyReachableItems();
-    for (int i = 0; i < m_visibleItems.count(); i++) {
-        ProgressItemWidget *itemWidget = m_itemToItemWidget.value(m_visibleItems.at(i));
+    for (WizardProgressItem *progressItem : std::as_const(m_visibleItems)) {
+        ProgressItemWidget *itemWidget = m_itemToItemWidget.value(progressItem);
         m_itemWidgetLayout->addWidget(itemWidget);
         itemWidget->setVisible(true);
     }
@@ -265,14 +242,11 @@ void LinearProgressWidget::updateProgress()
 
     QList<WizardProgressItem *> visitedItems = m_wizardProgress->visitedItems();
 
-    QMap<WizardProgressItem *, ProgressItemWidget *>::ConstIterator it = m_itemToItemWidget.constBegin();
-    QMap<WizardProgressItem *, ProgressItemWidget *>::ConstIterator itEnd = m_itemToItemWidget.constEnd();
-    while (it != itEnd) {
+    for (auto it = m_itemToItemWidget.cbegin(); it != m_itemToItemWidget.cend(); ++it) {
         WizardProgressItem *item = it.key();
         ProgressItemWidget *itemWidget = it.value();
         itemWidget->setEnabled(visitedItems.contains(item));
         itemWidget->setIndicatorVisible(false);
-        ++it;
     }
 
     WizardProgressItem *currentItem = m_wizardProgress->currentItem();
@@ -305,10 +279,11 @@ public:
     bool m_automaticProgressCreation = true;
     WizardProgress *m_wizardProgress = nullptr;
     QSet<QString> m_fieldNames;
+    bool m_skipForSubproject = false;
 };
 
-Wizard::Wizard(QWidget *parent, Qt::WindowFlags flags) :
-    QWizard(parent, flags), d_ptr(new WizardPrivate)
+Wizard::Wizard(Qt::WindowFlags flags)
+    : QWizard(dialogParent(), flags), d_ptr(new WizardPrivate)
 {
     d_ptr->m_wizardProgress = new WizardProgress(this);
     connect(this, &QWizard::currentIdChanged, this, &Wizard::_q_currentPageChanged);
@@ -318,7 +293,7 @@ Wizard::Wizard(QWidget *parent, Qt::WindowFlags flags) :
     setOption(QWizard::NoCancelButton, false);
     setOption(QWizard::NoDefaultButton, false);
     setOption(QWizard::NoBackButtonOnStartPage, true);
-    if (!Utils::creatorTheme()->preferredStyles().isEmpty())
+    if (!creatorTheme()->preferredStyles().isEmpty())
         setWizardStyle(QWizard::ModernStyle);
 
     if (HostOsInfo::isMacHost()) {
@@ -385,17 +360,17 @@ QSet<QString> Wizard::fieldNames() const
 QHash<QString, QVariant> Wizard::variables() const
 {
     QHash<QString, QVariant> result;
-    foreach (const QString &f, fieldNames()) {
+    const QSet<QString> fields = fieldNames();
+    for (const QString &f : fields)
         result.insert(f, field(f));
-    }
     return result;
 }
 
 QString typeOf(const QVariant &v)
 {
     QString result;
-    switch (v.type()) {
-    case QVariant::Map:
+    switch (v.typeId()) {
+    case QMetaType::QVariantMap:
         result = QLatin1String("Object");
         break;
     default:
@@ -408,9 +383,8 @@ void Wizard::showVariables()
 {
     QString result = QLatin1String("<table>\n  <tr><td>Key</td><td>Type</td><td>Value</td><td>Eval</td></tr>\n");
     QHash<QString, QVariant> vars = variables();
-    QList<QString> keys = vars.keys();
-    sort(keys);
-    foreach (const QString &key, keys) {
+    const QList<QString> keys = sorted(vars.keys());
+    for (const QString &key : keys) {
         const QVariant &v = vars.value(key);
         result += QLatin1String("  <tr><td>")
                 + key + QLatin1String("</td><td>")
@@ -555,8 +529,30 @@ void Wizard::_q_pageRemoved(int pageId)
     d->m_wizardProgress->removeItem(item);
 }
 
+void Wizard::setSkipForSubprojects(bool skip)
+{
+    Q_D(Wizard);
+    d->m_skipForSubproject = skip;
+}
 
+int Wizard::nextId() const
+{
+    Q_D(const Wizard);
+    if (!d->m_skipForSubproject)
+        return QWizard::nextId();
 
+    const QList<int> allIds = pageIds();
+    int index = allIds.indexOf(currentId());
+    QTC_ASSERT(index > -1, return QWizard::nextId());
+
+    while (++index < allIds.size()) {
+        if (auto wp = qobject_cast<WizardPage *>(page(index))) {
+            if (!wp->skipForSubprojects())
+                return index;
+        }
+    }
+    return QWizard::nextId();
+}
 
 class WizardProgressPrivate
 {
@@ -566,15 +562,13 @@ class WizardProgressPrivate
 public:
     WizardProgressPrivate() = default;
 
-    bool isNextItem(WizardProgressItem *item, WizardProgressItem *nextItem) const;
+    static bool isNextItem(WizardProgressItem *item, WizardProgressItem *nextItem);
     // if multiple paths are possible the empty list is returned
     QList<WizardProgressItem *> singlePathBetween(WizardProgressItem *fromItem, WizardProgressItem *toItem) const;
     void updateReachableItems();
 
     QMap<int, WizardProgressItem *> m_pageToItem;
-    QMap<WizardProgressItem *, WizardProgressItem *> m_itemToItem;
-
-    QList<WizardProgressItem *> m_items;
+    QSet<WizardProgressItem *> m_items;
 
     QList<WizardProgressItem *> m_visitedItems;
     QList<WizardProgressItem *> m_reachableItems;
@@ -583,6 +577,21 @@ public:
     WizardProgressItem *m_startItem = nullptr;
 };
 
+inline QDebug &operator<<(QDebug &debug, const WizardProgressPrivate &progress)
+{
+    debug << "items:" << progress.m_items.size()
+          << "; visited:" << progress.m_visitedItems.size()
+          << "; reachable:" << progress.m_reachableItems.size();
+
+    return debug;
+}
+
+QDebug &operator<<(QDebug &debug, const WizardProgress &progress)
+{
+    debug << "WizardProgress{_: " << *progress.d_ptr << "}";
+    return debug;
+}
+
 class WizardProgressItemPrivate
 {
     WizardProgressItem *q_ptr;
@@ -597,7 +606,23 @@ public:
     WizardProgressItem *m_nextShownItem;
 };
 
-bool WizardProgressPrivate::isNextItem(WizardProgressItem *item, WizardProgressItem *nextItem) const
+inline QDebug &operator<<(QDebug &debug, const WizardProgressItemPrivate &item)
+{
+    debug << "title:" << item.m_title
+          << "; word wrap:" << item.m_titleWordWrap
+          << "; progress:" << *item.m_wizardProgress
+          << "; pages:" << item.m_pages;
+
+    return debug;
+}
+
+QDebug &operator<<(QDebug &debug, const WizardProgressItem &item)
+{
+    debug << "WizardProgressItem{_: " << *item.d_ptr << "}";
+    return debug;
+}
+
+bool WizardProgressPrivate::isNextItem(WizardProgressItem *item, WizardProgressItem *nextItem)
 {
     QHash<WizardProgressItem *, bool> visitedItems;
     QList<WizardProgressItem *> workingItems = item->nextItems();
@@ -619,7 +644,7 @@ QList<WizardProgressItem *> WizardProgressPrivate::singlePathBetween(WizardProgr
     if (!item)
         item = m_startItem;
     if (!item)
-        return QList<WizardProgressItem *>();
+        return {};
 
     // Optimization. It is workaround for case A->B, B->C, A->C where "from" is A and "to" is C.
     // When we had X->A in addition and "from" was X and "to" was C, this would not work
@@ -627,12 +652,12 @@ QList<WizardProgressItem *> WizardProgressPrivate::singlePathBetween(WizardProgr
     if (item->nextItems().contains(toItem))
         return {toItem};
 
-    QHash<WizardProgressItem *, QHash<WizardProgressItem *, bool> > visitedItemsToParents;
-    QList<QPair<WizardProgressItem *, WizardProgressItem *> > workingItems; // next to prev item
+    QHash<WizardProgressItem *, QHash<WizardProgressItem *, bool>> visitedItemsToParents;
+    QList<QPair<WizardProgressItem *, WizardProgressItem *>> workingItems; // next to prev item
 
-    QList<WizardProgressItem *> items = item->nextItems();
-    for (int i = 0; i < items.count(); i++)
-        workingItems.append(qMakePair(items.at(i), item));
+    const QList<WizardProgressItem *> items = item->nextItems();
+    for (WizardProgressItem *i : items)
+        workingItems.push_back({i, item});
 
     while (!workingItems.isEmpty()) {
         QPair<WizardProgressItem *, WizardProgressItem *> workingItem = workingItems.takeFirst();
@@ -642,20 +667,20 @@ QList<WizardProgressItem *> WizardProgressPrivate::singlePathBetween(WizardProgr
         if (parents.count() > 1)
             continue;
 
-        QList<WizardProgressItem *> items = workingItem.first->nextItems();
-        for (int i = 0; i < items.count(); i++)
-            workingItems.append(qMakePair(items.at(i), workingItem.first));
+        const QList<WizardProgressItem *> items = workingItem.first->nextItems();
+        for (WizardProgressItem *i : items)
+            workingItems.push_back({i, workingItem.first});
     }
 
     QList<WizardProgressItem *> path;
 
     WizardProgressItem *it = toItem;
-    QHash<WizardProgressItem *, QHash<WizardProgressItem *, bool> >::ConstIterator itItem = visitedItemsToParents.constFind(it);
-    QHash<WizardProgressItem *, QHash<WizardProgressItem *, bool> >::ConstIterator itEnd = visitedItemsToParents.constEnd();
+    auto itItem = visitedItemsToParents.constFind(it);
+    const auto itEnd = visitedItemsToParents.constEnd();
     while (itItem != itEnd) {
         path.prepend(itItem.key());
         if (itItem.value().count() != 1)
-            return QList<WizardProgressItem *>();
+            return {};
         it = itItem.value().constBegin().key();
         if (it == item)
             return path;
@@ -693,12 +718,8 @@ WizardProgress::~WizardProgress()
 {
     Q_D(WizardProgress);
 
-    QMap<WizardProgressItem *, WizardProgressItem *>::ConstIterator it = d->m_itemToItem.constBegin();
-    QMap<WizardProgressItem *, WizardProgressItem *>::ConstIterator itEnd = d->m_itemToItem.constEnd();
-    while (it != itEnd) {
-        delete it.key();
-        ++it;
-    }
+    for (WizardProgressItem *item : std::as_const(d->m_items))
+        delete item;
     delete d_ptr;
 }
 
@@ -707,7 +728,7 @@ WizardProgressItem *WizardProgress::addItem(const QString &title)
     Q_D(WizardProgress);
 
     auto item = new WizardProgressItem(this, title);
-    d->m_itemToItem.insert(item, item);
+    d->m_items.insert(item);
     emit itemAdded(item);
     return item;
 }
@@ -716,8 +737,8 @@ void WizardProgress::removeItem(WizardProgressItem *item)
 {
     Q_D(WizardProgress);
 
-    QMap<WizardProgressItem *, WizardProgressItem *>::iterator it = d->m_itemToItem.find(item);
-    if (it == d->m_itemToItem.end()) {
+    const auto it = d->m_items.constFind(item);
+    if (it == d->m_items.constEnd()) {
         qWarning("WizardProgress::removePage: Item is not a part of the wizard");
         return;
     }
@@ -749,7 +770,7 @@ void WizardProgress::removeItem(WizardProgressItem *item)
     QList<int> pages = item->pages();
     for (int i = 0; i < pages.count(); i++)
         d->m_pageToItem.remove(pages.at(i));
-    d->m_itemToItem.erase(it);
+    d->m_items.erase(it);
     delete item;
 }
 
@@ -757,8 +778,8 @@ void WizardProgress::removePage(int pageId)
 {
     Q_D(WizardProgress);
 
-    QMap<int, WizardProgressItem *>::iterator it = d->m_pageToItem.find(pageId);
-    if (it == d->m_pageToItem.end()) {
+    const auto it = d->m_pageToItem.constFind(pageId);
+    if (it == d->m_pageToItem.constEnd()) {
         qWarning("WizardProgress::removePage: page is not a part of the wizard");
         return;
     }
@@ -767,7 +788,7 @@ void WizardProgress::removePage(int pageId)
     item->d_ptr->m_pages.removeOne(pageId);
 }
 
-QList<int> WizardProgress::pages(WizardProgressItem *item) const
+QList<int> WizardProgress::pages(WizardProgressItem *item)
 {
     return item->pages();
 }
@@ -786,11 +807,11 @@ WizardProgressItem *WizardProgress::currentItem() const
     return d->m_currentItem;
 }
 
-QList<WizardProgressItem *> WizardProgress::items() const
+QSet<WizardProgressItem *> WizardProgress::items() const
 {
     Q_D(const WizardProgress);
 
-    return d->m_itemToItem.keys();
+    return d->m_items;
 }
 
 WizardProgressItem *WizardProgress::startItem() const
@@ -931,7 +952,7 @@ void WizardProgressItem::setNextItems(const QList<WizardProgressItem *> &items)
     // check if we create cycle
     for (int i = 0; i < items.count(); i++) {
         WizardProgressItem *nextItem = items.at(i);
-        if (nextItem == this || d->m_wizardProgress->d_ptr->isNextItem(nextItem, this)) {
+        if (nextItem == this || WizardProgressPrivate::isNextItem(nextItem, this)) {
             qWarning("WizardProgress::setNextItems: Setting one of the next items would create a cycle");
             return;
         }
