@@ -48,7 +48,7 @@ class CppEditorDocumentHandleImpl : public CppEditorDocumentHandle
 public:
     CppEditorDocumentHandleImpl(CppEditorDocument *cppEditorDocument)
         : m_cppEditorDocument(cppEditorDocument)
-        , m_registrationFilePath(cppEditorDocument->filePath().toString())
+        , m_registrationFilePath(cppEditorDocument->filePath().toUrlishString())
     {
         CppModelManager::registerCppEditorDocument(this);
     }
@@ -79,6 +79,8 @@ CppEditorDocument::CppEditorDocument()
 {
     setId(CppEditor::Constants::CPPEDITOR_ID);
     resetSyntaxHighlighter([] { return new CppHighlighter(); });
+    connect(syntaxHighlighter(), &SyntaxHighlighter::finished,
+            this, &CppEditorDocument::applyIfdefedOutBlocks);
 
     ICodeStylePreferencesFactory *factory
         = TextEditorSettings::codeStyleFactory(Constants::CPP_SETTINGS_ID);
@@ -171,6 +173,12 @@ void CppEditorDocument::applyFontSettings()
         m_processor->semanticRehighlight();
 }
 
+void CppEditorDocument::slotCodeStyleSettingsChanged()
+{
+    QtStyleCodeFormatter formatter;
+    formatter.invalidateCache(document());
+}
+
 void CppEditorDocument::invalidateFormatterCache()
 {
     QtStyleCodeFormatter formatter;
@@ -210,7 +218,7 @@ void CppEditorDocument::reparseWithPreferredParseContext(const QString &parseCon
     setPreferredParseContext(parseContextId);
 
     // Remember the setting
-    const Key key = Constants::PREFERRED_PARSE_CONTEXT + keyFromString(filePath().toString());
+    const Key key = Constants::PREFERRED_PARSE_CONTEXT + keyFromString(filePath().toUrlishString());
     Core::SessionManager::setValue(key, parseContextId);
 
     // Reprocess
@@ -277,7 +285,7 @@ void CppEditorDocument::applyPreferredParseContextFromSettings()
     if (filePath().isEmpty())
         return;
 
-    const Key key = Constants::PREFERRED_PARSE_CONTEXT + keyFromString(filePath().toString());
+    const Key key = Constants::PREFERRED_PARSE_CONTEXT + keyFromString(filePath().toUrlishString());
     const QString parseContextId = Core::SessionManager::value(key).toString();
 
     setPreferredParseContext(parseContextId);
@@ -288,7 +296,7 @@ void CppEditorDocument::applyExtraPreprocessorDirectivesFromSettings()
     if (filePath().isEmpty())
         return;
 
-    const Key key = Constants::EXTRA_PREPROCESSOR_DIRECTIVES + keyFromString(filePath().toString());
+    const Key key = Constants::EXTRA_PREPROCESSOR_DIRECTIVES + keyFromString(filePath().toUrlishString());
     const QByteArray directives = Core::SessionManager::value(key).toString().toUtf8();
 
     setExtraPreprocessorDirectives(directives);
@@ -310,14 +318,14 @@ void CppEditorDocument::setExtraPreprocessorDirectives(const QByteArray &directi
 
 void CppEditorDocument::setIfdefedOutBlocks(const QList<TextEditor::BlockRange> &blocks)
 {
-    if (syntaxHighlighter() && !syntaxHighlighter()->syntaxHighlighterUpToDate()) {
-        connect(syntaxHighlighter(),
-            &SyntaxHighlighter::finished,
-            this,
-            [this, blocks] { setIfdefedOutBlocks(blocks); },
-            Qt::SingleShotConnection);
+    m_ifdefedOutBlocks = blocks;
+    applyIfdefedOutBlocks();
+}
+
+void CppEditorDocument::applyIfdefedOutBlocks()
+{
+    if (!syntaxHighlighter() || !syntaxHighlighter()->syntaxHighlighterUpToDate())
         return;
-    }
 
     auto documentLayout = qobject_cast<TextDocumentLayout*>(document()->documentLayout());
     QTC_ASSERT(documentLayout, return);
@@ -328,8 +336,8 @@ void CppEditorDocument::setIfdefedOutBlocks(const QList<TextEditor::BlockRange>
     int previousBraceDepth = 0;
     while (block.isValid()) {
         bool resetToPrevious = false;
-        if (rangeNumber < blocks.size()) {
-            const BlockRange &range = blocks.at(rangeNumber);
+        if (rangeNumber < m_ifdefedOutBlocks.size()) {
+            const BlockRange &range = m_ifdefedOutBlocks.at(rangeNumber);
             if (block.position() >= range.first()
                 && ((block.position() + block.length() - 1) <= range.last() || !range.last())) {
                 TextDocumentLayout::setIfdefedOut(block);
@@ -366,6 +374,10 @@ void CppEditorDocument::setIfdefedOutBlocks(const QList<TextEditor::BlockRange>
 
     if (needUpdate)
         documentLayout->requestUpdate();
+
+#ifdef WITH_TESTS
+    emit ifdefedOutBlocksApplied();
+#endif
 }
 
 void CppEditorDocument::setPreferredParseContext(const QString &parseContextId)
