@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
@@ -31,15 +9,26 @@
 #include <QMutex>
 #include <QObject>
 #include <QUrl>
-#include <QStandardItemModel>
 
-QT_FORWARD_DECLARE_CLASS(QHelpEngine)
+QT_BEGIN_NAMESPACE
+class QHelpFilterEngine;
+class QHelpEngine;
+QT_END_NAMESPACE
 
 class BookmarkManager;
 
 namespace Help {
 namespace Internal {
 
+class HelpViewer;
+
+struct HelpViewerFactory
+{
+    QByteArray id;
+    QString displayName;
+    std::function<HelpViewer *()> create;
+};
+
 class LocalHelpManager : public QObject
 {
     Q_OBJECT
@@ -57,8 +46,8 @@ public:
         ShowLastPages = 2,
     };
 
-    LocalHelpManager(QObject *parent = 0);
-    ~LocalHelpManager();
+    LocalHelpManager(QObject *parent = nullptr);
+    ~LocalHelpManager() override;
 
     static LocalHelpManager *instance();
 
@@ -69,6 +58,12 @@ public:
     static QFont fallbackFont();
     static void setFallbackFont(const QFont &font);
 
+    static int fontZoom();
+    static int setFontZoom(int percentage);
+
+    static bool antialias();
+    static void setAntialias(bool on);
+
     static StartOption startOption();
     static void setStartOption(StartOption option);
 
@@ -78,50 +73,60 @@ public:
     static bool returnOnClose();
     static void setReturnOnClose(bool returnOnClose);
 
+    static bool isScrollWheelZoomingEnabled();
+    static void setScrollWheelZoomingEnabled(bool enabled);
+
     static QStringList lastShownPages();
     static void setLastShownPages(const QStringList &pages);
 
-    static QList<float> lastShownPagesZoom();
-    static void setLastShownPagesZoom(const QList<float> &zoom);
-
     static int lastSelectedTab();
     static void setLastSelectedTab(int index);
 
+    static HelpViewerFactory defaultViewerBackend();
+    static QVector<HelpViewerFactory> viewerBackends();
+    static HelpViewerFactory viewerBackend();
+    static void setViewerBackendId(const QByteArray &id);
+    static QByteArray viewerBackendId();
+
     static void setupGuiHelpEngine();
     static void setEngineNeedsUpdate();
 
     static QHelpEngine& helpEngine();
     static BookmarkManager& bookmarkManager();
 
-    static bool isValidUrl(const QString &link);
-
     static QByteArray loadErrorMessage(const QUrl &url, const QString &errorString);
     Q_INVOKABLE static Help::Internal::LocalHelpManager::HelpData helpData(const QUrl &url);
 
-    static QAbstractItemModel *filterModel();
-    static void setFilterIndex(int index);
-    static int filterIndex();
+    static QHelpFilterEngine *filterEngine();
 
-    static void updateFilterModel();
+    static bool canOpenOnlineHelp(const QUrl &url);
+    static bool openOnlineHelp(const QUrl &url);
+    static bool isQtUrl(const QUrl &url);
+    static void openQtUrl(const QUrl &url);
+
+    static QMultiMap<QString, QUrl> linksForKeyword(const QString &keyword);
+
+    static void addOnlineHelpHandler(const Core::HelpManager::OnlineHelpHandler &handler);
 
 signals:
-    void filterIndexChanged(int index);
     void fallbackFontChanged(const QFont &font);
+    void fontZoomChanged(int percentage);
+    void antialiasChanged(bool on);
     void returnOnCloseChanged();
+    void scrollWheelZoomingEnabledChanged(bool enabled);
+    void contextHelpOptionChanged(Core::HelpManager::HelpViewerLocation option);
 
 private:
     static bool m_guiNeedsSetup;
     static bool m_needsCollectionFile;
 
-    static QStandardItemModel *m_filterModel;
-    static QString m_currentFilter;
-    static int m_currentFilterIndex;
-
     static QMutex m_guiMutex;
     static QHelpEngine *m_guiEngine;
 
     static QMutex m_bkmarkMutex;
     static BookmarkManager *m_bookmarkManager;
+
+    static QList<Core::HelpManager::OnlineHelpHandler> m_onlineHelpHandlerList;
 };
 
 }   // Internal
