@@ -15,6 +15,8 @@
 
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectmanager.h>
+#include <projectexplorer/projectpanelfactory.h>
+#include <projectexplorer/projectsettingswidget.h>
 
 #include <texteditor/plaintexteditorfactory.h>
 #include <texteditor/textmark.h>
@@ -23,7 +25,9 @@
 #include <utils/delegates.h>
 #include <utils/fancylineedit.h>
 #include <utils/jsontreeitem.h>
+#include <utils/layoutbuilder.h>
 #include <utils/macroexpander.h>
+#include <utils/mimeconstants.h>
 #include <utils/stringutils.h>
 #include <utils/utilsicons.h>
 #include <utils/variablechooser.h>
@@ -36,6 +40,7 @@
 #include <QDialogButtonBox>
 #include <QDir>
 #include <QFileInfo>
+#include <QFormLayout>
 #include <QGroupBox>
 #include <QHeaderView>
 #include <QJsonDocument>
@@ -48,6 +53,7 @@
 #include <QStringListModel>
 #include <QToolButton>
 #include <QTreeView>
+#include <QTreeWidget>
 
 constexpr char typeIdKey[] = "typeId";
 constexpr char nameKey[] = "name";
@@ -66,6 +72,7 @@ constexpr char typedClientsKey[] = "typedClients";
 constexpr char outlineSortedKey[] = "outlineSorted";
 constexpr char mimeType[] = "application/language.client.setting";
 
+using namespace ProjectExplorer;
 using namespace Utils;
 
 namespace LanguageClient {
@@ -94,7 +101,7 @@ public:
 
     void reset(const QList<BaseSettings *> &settings);
     QList<BaseSettings *> settings() const { return m_settings; }
-    int insertSettings(BaseSettings *settings);
+    QModelIndex insertSettings(BaseSettings *settings);
     void enableSetting(const QString &id, bool enable = true);
     QList<BaseSettings *> removed() const { return m_removed; }
     BaseSettings *settingForIndex(const QModelIndex &index) const;
@@ -106,6 +113,52 @@ private:
     QList<BaseSettings *> m_removed;
 };
 
+class FilterProxy final : public QSortFilterProxyModel
+{
+public:
+    FilterProxy(LanguageClientSettingsModel &sourceModel)
+        : m_settings(sourceModel)
+    {
+        setSourceModel(&sourceModel);
+    }
+
+    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const final
+    {
+        const QModelIndex index = sourceModel()->index(sourceRow, 0, sourceParent);
+        const BaseSettings *setting
+            = static_cast<LanguageClientSettingsModel *>(sourceModel())->settingForIndex(index);
+        return setting && setting->m_showInSettings;
+    }
+
+    void reset(QList<BaseSettings *> settings)
+    {
+        m_settings.reset(settings);
+        invalidateFilter();
+    }
+
+    QModelIndex insertSettings(BaseSettings *settings)
+    {
+        const auto idx = m_settings.insertSettings(settings);
+        invalidateFilter();
+        return mapFromSource(idx);
+    }
+
+    BaseSettings *settingForIndex(const QModelIndex &index) const
+    {
+        return m_settings.settingForIndex(mapToSource(index));
+    }
+
+    QModelIndex indexForSetting(BaseSettings *setting) const
+    {
+        return mapFromSource(m_settings.indexForSetting(setting));
+    }
+
+    QList<BaseSettings *> removed() const { return m_settings.removed(); }
+
+private:
+    LanguageClientSettingsModel &m_settings;
+};
+
 class LanguageClientSettingsPageWidget : public Core::IOptionsPageWidget
 {
 public:
@@ -131,7 +184,7 @@ public:
         m_settings.reset(LanguageClientManager::currentSettings());
         resetCurrentSettings(row);
     }
-    void finish()
+    void finish() override
     {
         m_settings.reset(LanguageClientManager::currentSettings());
         m_changedSettings.clear();
@@ -147,7 +200,7 @@ private:
     void addItem(const Utils::Id &clientTypeId);
     void deleteItem();
 
-    LanguageClientSettingsModel &m_settings;
+    FilterProxy m_settings;
     QSet<QString> &m_changedSettings;
 };
 
@@ -165,6 +218,7 @@ LanguageClientSettingsPageWidget::LanguageClientSettingsPageWidget(LanguageClien
 {
     auto mainLayout = new QVBoxLayout();
     auto layout = new QHBoxLayout();
+
     m_view->setModel(&m_settings);
     m_view->setHeaderHidden(true);
     m_view->setSelectionMode(QAbstractItemView::SingleSelection);
@@ -177,10 +231,12 @@ LanguageClientSettingsPageWidget::LanguageClientSettingsPageWidget(LanguageClien
             this, &LanguageClientSettingsPageWidget::currentChanged);
     auto buttonLayout = new QVBoxLayout();
     auto addButton = new QPushButton(Tr::tr("&Add"));
-    auto addMenu = new QMenu;
+    auto addMenu = new QMenu(this);
     addMenu->clear();
     for (const ClientType &type : clientTypes()) {
-        auto action = new QAction(type.name);
+        if (!type.userAddable)
+            continue;
+        auto action = new QAction(type.name, this);
         connect(action, &QAction::triggered, this, [this, id = type.id]() { addItem(id); });
         addMenu->addAction(action);
     }
@@ -228,7 +284,7 @@ void LanguageClientSettingsPageWidget::resetCurrentSettings(int row)
 
     m_currentSettings.setting = nullptr;
     m_currentSettings.widget = nullptr;
-    m_view->setCurrentIndex(m_settings.index(row));
+    m_view->setCurrentIndex(m_settings.index(row, 0));
 }
 
 void LanguageClientSettingsPageWidget::applyCurrentSettings()
@@ -238,7 +294,7 @@ void LanguageClientSettingsPageWidget::applyCurrentSettings()
 
     if (m_currentSettings.setting->applyFromSettingsWidget(m_currentSettings.widget)) {
         auto index = m_settings.indexForSetting(m_currentSettings.setting);
-        emit m_settings.dataChanged(index, index);
+        emit m_settings.sourceModel()->dataChanged(index, index);
     }
 }
 
@@ -256,7 +312,7 @@ void LanguageClientSettingsPageWidget::addItem(const Utils::Id &clientTypeId)
 {
     auto newSettings = generateSettings(clientTypeId);
     QTC_ASSERT(newSettings, return);
-    m_view->setCurrentIndex(m_settings.index(m_settings.insertSettings(newSettings)));
+    m_view->setCurrentIndex(m_settings.insertSettings(newSettings));
 }
 
 void LanguageClientSettingsPageWidget::deleteItem()
@@ -265,7 +321,7 @@ void LanguageClientSettingsPageWidget::deleteItem()
     if (!index.isValid())
         return;
 
-    m_settings.removeRows(index.row());
+    m_settings.removeRow(index.row());
 }
 
 class LanguageClientSettingsPage : public Core::IOptionsPage
@@ -290,8 +346,6 @@ LanguageClientSettingsPage::LanguageClientSettingsPage()
     setId(Constants::LANGUAGECLIENT_SETTINGS_PAGE);
     setDisplayName(Tr::tr("General"));
     setCategory(Constants::LANGUAGECLIENT_SETTINGS_CATEGORY);
-    setDisplayCategory(Tr::tr(Constants::LANGUAGECLIENT_SETTINGS_TR));
-    setCategoryIconPath(":/languageclient/images/settingscategory_languageclient.png");
     setWidgetCreator([this] { return new LanguageClientSettingsPageWidget(m_model, m_changedSettings); });
     QObject::connect(&m_model, &LanguageClientSettingsModel::dataChanged, [this](const QModelIndex &index) {
         if (BaseSettings *setting = m_model.settingForIndex(index))
@@ -301,7 +355,9 @@ LanguageClientSettingsPage::LanguageClientSettingsPage()
 
 void LanguageClientSettingsPage::init()
 {
-    m_model.reset(LanguageClientSettings::fromSettings(Core::ICore::settings()));
+    QList<BaseSettings *> newList = LanguageClientSettings::fromSettings(Core::ICore::settings());
+    m_model.reset(newList);
+    qDeleteAll(newList);
 }
 
 QList<BaseSettings *> LanguageClientSettingsPage::settings() const
@@ -447,13 +503,13 @@ void LanguageClientSettingsModel::reset(const QList<BaseSettings *> &settings)
     endResetModel();
 }
 
-int LanguageClientSettingsModel::insertSettings(BaseSettings *settings)
+QModelIndex LanguageClientSettingsModel::insertSettings(BaseSettings *settings)
 {
     int row = rowCount();
     beginInsertRows(QModelIndex(), row, row);
     m_settings.insert(row, settings);
     endInsertRows();
-    return row;
+    return createIndex(row, 0, settings);
 }
 
 void LanguageClientSettingsModel::enableSetting(const QString &id, bool enable)
@@ -532,19 +588,41 @@ bool BaseSettings::isValid() const
     return !m_name.isEmpty();
 }
 
+bool BaseSettings::isValidOnProject(ProjectExplorer::Project *) const
+{
+    return isValid();
+}
+
 Client *BaseSettings::createClient() const
 {
     return createClient(static_cast<ProjectExplorer::Project *>(nullptr));
 }
 
+
+bool BaseSettings::isEnabledOnProject(ProjectExplorer::Project *project) const
+{
+    if (project) {
+        LanguageClient::ProjectSettings settings(project);
+        if (settings.enabledSettings().contains(m_id))
+            return true;
+        if (settings.disabledSettings().contains(m_id))
+            return false;
+    }
+    return m_enabled;
+}
+
 Client *BaseSettings::createClient(ProjectExplorer::Project *project) const
 {
-    if (!isValid() || !m_enabled)
+    if (!isValidOnProject(project) || !isEnabledOnProject(project))
         return nullptr;
     BaseClientInterface *interface = createInterface(project);
     QTC_ASSERT(interface, return nullptr);
     auto *client = createClient(interface);
-    client->setName(Utils::globalMacroExpander()->expand(m_name));
+    QTC_ASSERT(client, return nullptr);
+
+    if (client->name().isEmpty())
+        client->setName(Utils::globalMacroExpander()->expand(m_name));
+
     client->setSupportedLanguage(m_languageFilter);
     client->setInitializationOptions(initializationOptions());
     client->setActivateDocumentAutomatically(true);
@@ -553,19 +631,13 @@ Client *BaseSettings::createClient(ProjectExplorer::Project *project) const
     return client;
 }
 
-BaseClientInterface *BaseSettings::createInterface(ProjectExplorer::Project *) const
-{
-    return nullptr;
-}
-
 Client *BaseSettings::createClient(BaseClientInterface *interface) const
 {
     return new Client(interface);
 }
 
-Store BaseSettings::toMap() const
+void BaseSettings::toMap(Store &map) const
 {
-    Store map;
     map.insert(typeIdKey, m_settingsTypeId.toSetting());
     map.insert(nameKey, m_name);
     map.insert(idKey, m_id);
@@ -575,7 +647,6 @@ Store BaseSettings::toMap() const
     map.insert(filePatternKey, m_languageFilter.filePattern);
     map.insert(initializationOptionsKey, m_initializationOptions);
     map.insert(configurationKey, m_configuration);
-    return map;
 }
 
 void BaseSettings::fromMap(const Store &map)
@@ -604,6 +675,27 @@ void LanguageClientSettings::init()
     LanguageClientManager::applySettings();
 }
 
+QList<Utils::Store> LanguageClientSettings::storesBySettingsType(Utils::Id settingsTypeId)
+{
+    QList<Utils::Store> result;
+
+    QtcSettings *settingsIn = Core::ICore::settings();
+    settingsIn->beginGroup(settingsGroupKey);
+
+    for (const QVariantList &varList :
+         {settingsIn->value(clientsKey).toList(), settingsIn->value(typedClientsKey).toList()}) {
+        for (const QVariant &var : varList) {
+            const Store store = storeFromVariant(var);
+            if (settingsTypeId == Id::fromSetting(store.value(typeIdKey)))
+                result << store;
+        }
+    }
+
+    settingsIn->endGroup();
+
+    return result;
+}
+
 QList<BaseSettings *> LanguageClientSettings::fromSettings(QtcSettings *settingsIn)
 {
     settingsIn->beginGroup(settingsGroupKey);
@@ -659,14 +751,29 @@ void LanguageClientSettings::toSettings(QtcSettings *settings,
     settings->beginGroup(settingsGroupKey);
     auto transform = [](const QList<BaseSettings *> &settings) {
         return Utils::transform(settings, [](const BaseSettings *setting) {
-            return variantFromStore(setting->toMap());
+            Store store;
+            setting->toMap(store);
+            return variantFromStore(store);
         });
     };
     auto isStdioSetting = Utils::equal(&BaseSettings::m_settingsTypeId,
                                        Utils::Id(Constants::LANGUAGECLIENT_STDIO_SETTINGS_ID));
     auto [stdioSettings, typedSettings] = Utils::partition(languageClientSettings, isStdioSetting);
     settings->setValue(clientsKey, transform(stdioSettings));
-    settings->setValue(typedClientsKey, transform(typedSettings));
+
+    // write back typed settings for unregistered client types
+    QVariantList typedSettingsVariant;
+    for (const QVariant &var : settings->value(typedClientsKey).toList()) {
+        const Store map = storeFromVariant(var);
+        const Id typeId = Id::fromSetting(map.value(typeIdKey));
+        const QString id = map.value(idKey).toString();
+        if (typeId.isValid() && !clientTypes().contains(typeId)
+            && !Utils::anyOf(typedSettings, Utils::equal(&BaseSettings::m_id, id)))
+            typedSettingsVariant << var;
+    }
+
+    typedSettingsVariant << transform(typedSettings);
+    settings->setValue(typedClientsKey, typedSettingsVariant);
     settings->endGroup();
 }
 
@@ -714,12 +821,11 @@ bool StdIOSettings::isValid() const
     return BaseSettings::isValid() && !m_executable.isEmpty();
 }
 
-Store StdIOSettings::toMap() const
+void StdIOSettings::toMap(Store &map) const
 {
-    Store map = BaseSettings::toMap();
+    BaseSettings::toMap(map);
     map.insert(executableKey, m_executable.toSettings());
     map.insert(argumentsKey, m_arguments);
-    return map;
 }
 
 void StdIOSettings::fromMap(const Store &map)
@@ -783,71 +889,73 @@ static QString startupBehaviorString(BaseSettings::StartBehavior behavior)
     return {};
 }
 
-BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *parent)
+BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *parent,
+                                       Layouting::LayoutModifier additionalItems)
     : QWidget(parent)
     , m_name(new QLineEdit(settings->m_name, this))
     , m_mimeTypes(new QLabel(settings->m_languageFilter.mimeTypes.join(filterSeparator), this))
-    , m_filePattern(new QLineEdit(settings->m_languageFilter.filePattern.join(filterSeparator), this))
+    , m_filePattern(
+          new QLineEdit(settings->m_languageFilter.filePattern.join(filterSeparator), this))
     , m_startupBehavior(new QComboBox)
     , m_initializationOptions(new Utils::FancyLineEdit(this))
 {
-    int row = 0;
-    auto *mainLayout = new QGridLayout;
+    using namespace Layouting;
 
-    mainLayout->addWidget(new QLabel(Tr::tr("Name:")), row, 0);
-    mainLayout->addWidget(m_name, row, 1);
     auto chooser = new Utils::VariableChooser(this);
     chooser->addSupportedWidget(m_name);
+    chooser->addSupportedWidget(m_initializationOptions);
 
-    mainLayout->addWidget(new QLabel(Tr::tr("Language:")), ++row, 0);
-    auto mimeLayout = new QHBoxLayout;
-    mimeLayout->addWidget(m_mimeTypes);
-    mimeLayout->addStretch();
     auto addMimeTypeButton = new QPushButton(Tr::tr("Set MIME Types..."), this);
-    mimeLayout->addWidget(addMimeTypeButton);
-    mainLayout->addLayout(mimeLayout, row, 1);
+    connect(
+        addMimeTypeButton, &QPushButton::pressed, this, &BaseSettingsWidget::showAddMimeTypeDialog);
+
     m_filePattern->setPlaceholderText(Tr::tr("File pattern"));
     m_filePattern->setToolTip(
         Tr::tr("List of file patterns.\nExample: *.cpp%1*.h").arg(filterSeparator));
-    mainLayout->addWidget(m_filePattern, ++row, 1);
 
-    mainLayout->addWidget(new QLabel(Tr::tr("Startup behavior:")), ++row, 0);
     for (int behavior = 0; behavior < BaseSettings::LastSentinel ; ++behavior)
         m_startupBehavior->addItem(startupBehaviorString(BaseSettings::StartBehavior(behavior)));
     m_startupBehavior->setCurrentIndex(settings->m_startBehavior);
-    mainLayout->addWidget(m_startupBehavior, row, 1);
 
+    m_initializationOptions->setValidationFunction(
+        [](Utils::FancyLineEdit *edit, QString *errorMessage) {
+            const QString value = Utils::globalMacroExpander()->expand(edit->text());
 
-    connect(addMimeTypeButton, &QPushButton::pressed,
-            this, &BaseSettingsWidget::showAddMimeTypeDialog);
+            if (value.isEmpty())
+                return true;
 
-    mainLayout->addWidget(new QLabel(Tr::tr("Initialization options:")), ++row, 0);
-    mainLayout->addWidget(m_initializationOptions, row, 1);
-    chooser->addSupportedWidget(m_initializationOptions);
-    m_initializationOptions->setValidationFunction([](Utils::FancyLineEdit *edit, QString *errorMessage) {
-        const QString value = Utils::globalMacroExpander()->expand(edit->text());
+            QJsonParseError parseInfo;
+            const QJsonDocument json = QJsonDocument::fromJson(value.toUtf8(), &parseInfo);
 
-        if (value.isEmpty())
+            if (json.isNull()) {
+                if (errorMessage)
+                    *errorMessage = Tr::tr("Failed to parse JSON at %1: %2")
+                                        .arg(parseInfo.offset)
+                                        .arg(parseInfo.errorString());
+                return false;
+            }
             return true;
-
-        QJsonParseError parseInfo;
-        const QJsonDocument json = QJsonDocument::fromJson(value.toUtf8(), &parseInfo);
-
-        if (json.isNull()) {
-            if (errorMessage)
-                *errorMessage = Tr::tr("Failed to parse JSON at %1: %2")
-                    .arg(parseInfo.offset)
-                    .arg(parseInfo.errorString());
-            return false;
-        }
-        return true;
-    });
+        });
     m_initializationOptions->setText(settings->m_initializationOptions);
     m_initializationOptions->setPlaceholderText(Tr::tr("Language server-specific JSON to pass via "
                                                    "\"initializationOptions\" field of \"initialize\" "
                                                    "request."));
 
-    setLayout(mainLayout);
+    // clang-format off
+    auto form = Form {
+        Tr::tr("Name:"), m_name, br,
+        Tr::tr("Language:"), Row { m_mimeTypes, st, addMimeTypeButton }, br,
+        Tr::tr("File pattern:"), m_filePattern, br,
+        Tr::tr("Startup behavior:"), m_startupBehavior, br,
+        Tr::tr("Initialization options:"), m_initializationOptions, br
+
+    };
+
+    if (additionalItems)
+        additionalItems(&form);
+
+    form.attachTo(this);
+    // clang-format on
 }
 
 QString BaseSettingsWidget::name() const
@@ -968,15 +1076,17 @@ StdIOSettingsWidget::StdIOSettingsWidget(const StdIOSettings *settings, QWidget
     , m_executable(new Utils::PathChooser(this))
     , m_arguments(new QLineEdit(settings->m_arguments, this))
 {
-    auto mainLayout = qobject_cast<QGridLayout *>(layout());
-    QTC_ASSERT(mainLayout, return);
-    const int baseRows = mainLayout->rowCount();
-    mainLayout->addWidget(new QLabel(Tr::tr("Executable:")), baseRows, 0);
-    mainLayout->addWidget(m_executable, baseRows, 1);
-    mainLayout->addWidget(new QLabel(Tr::tr("Arguments:")), baseRows + 1, 0);
+    using namespace Layouting;
+
     m_executable->setExpectedKind(Utils::PathChooser::ExistingCommand);
     m_executable->setFilePath(settings->m_executable);
-    mainLayout->addWidget(m_arguments, baseRows + 1, 1);
+
+    auto mainLayout = qobject_cast<QFormLayout *>(layout());
+    QTC_ASSERT(mainLayout, return);
+    int row = mainLayout->rowCount();
+
+    mainLayout->insertRow(row++, Tr::tr("Executable:"), m_executable);
+    mainLayout->insertRow(row++, Tr::tr("Arguments:"), m_arguments);
 
     auto chooser = new Utils::VariableChooser(this);
     chooser->addSupportedWidget(m_arguments);
@@ -992,10 +1102,16 @@ QString StdIOSettingsWidget::arguments() const
     return m_arguments->text();
 }
 
-bool LanguageFilter::isSupported(const Utils::FilePath &filePath, const QString &mimeType) const
+bool LanguageFilter::isSupported(const Utils::FilePath &filePath, const QString &mimeTypeName) const
 {
-    if (mimeTypes.contains(mimeType))
-        return true;
+    if (!mimeTypes.isEmpty()) {
+        const MimeType mimeType = Utils::mimeTypeForName(mimeTypeName);
+        if (Utils::anyOf(mimeTypes, [mimeType](const QString &supported) {
+                return mimeType.inherits(supported);
+            })) {
+            return true;
+        }
+    }
     if (filePattern.isEmpty() && filePath.isEmpty())
         return mimeTypes.isEmpty();
     const QRegularExpression::PatternOptions options
@@ -1006,7 +1122,7 @@ bool LanguageFilter::isSupported(const Utils::FilePath &filePath, const QString
                                   options);
     });
     return Utils::anyOf(regexps, [filePath](const QRegularExpression &reg){
-        return reg.match(filePath.toString()).hasMatch()
+        return reg.match(filePath.toUrlishString()).hasMatch()
                 || reg.match(filePath.fileName()).hasMatch();
     });
 }
@@ -1026,7 +1142,7 @@ bool LanguageFilter::operator!=(const LanguageFilter &other) const
     return this->filePattern != other.filePattern || this->mimeTypes != other.mimeTypes;
 }
 
-TextEditor::BaseTextEditor *jsonEditor()
+TextEditor::BaseTextEditor *createJsonEditor(QObject *parent)
 {
     using namespace TextEditor;
     using namespace Utils::Text;
@@ -1037,10 +1153,12 @@ TextEditor::BaseTextEditor *jsonEditor()
             break;
         delete editor;
     }
-    QTC_ASSERT(textEditor, textEditor = PlainTextEditorFactory::createPlainTextEditor());
+    QTC_ASSERT(textEditor, textEditor = createPlainTextEditor());
+    textEditor->setParent(parent);
+
     TextDocument *document = textEditor->textDocument();
     TextEditorWidget *widget = textEditor->editorWidget();
-    widget->configureGenericHighlighter(Utils::mimeTypeForName("application/json"));
+    widget->configureGenericHighlighter(mimeTypeForName(Utils::Constants::JSON_MIMETYPE));
     widget->setLineNumbersVisible(false);
     widget->setRevisionsVisible(false);
     widget->setCodeFoldingSupported(false);
@@ -1073,11 +1191,16 @@ TextEditor::BaseTextEditor *jsonEditor()
 }
 
 constexpr const char projectSettingsId[] = "LanguageClient.ProjectSettings";
+constexpr const char enabledSettingsId[] = "LanguageClient.EnabledSettings";
+constexpr const char disabledSettingsId[] = "LanguageClient.DisabledSettings";
 
 ProjectSettings::ProjectSettings(ProjectExplorer::Project *project)
     : m_project(project)
 {
+    QTC_ASSERT(project, return);
     m_json = m_project->namedSettings(projectSettingsId).toByteArray();
+    m_enabledSettings = m_project->namedSettings(enabledSettingsId).toStringList();
+    m_disabledSettings = m_project->namedSettings(disabledSettingsId).toStringList();
 }
 
 QJsonValue ProjectSettings::workspaceConfiguration() const
@@ -1097,6 +1220,7 @@ QByteArray ProjectSettings::json() const
 
 void ProjectSettings::setJson(const QByteArray &json)
 {
+    QTC_ASSERT(m_project, return);
     const QJsonValue oldConfig = workspaceConfiguration();
     m_json = json;
     m_project->setNamedSettings(projectSettingsId, m_json);
@@ -1105,30 +1229,140 @@ void ProjectSettings::setJson(const QByteArray &json)
         LanguageClientManager::updateWorkspaceConfiguration(m_project, newConfig);
 }
 
-ProjectSettingsWidget::ProjectSettingsWidget(ProjectExplorer::Project *project)
-    : m_settings(project)
+void ProjectSettings::enableSetting(const QString &id)
 {
-    setUseGlobalSettingsCheckBoxVisible(false);
-    setGlobalSettingsId(Constants::LANGUAGECLIENT_SETTINGS_PAGE);
-    setExpanding(true);
+    QTC_ASSERT(m_project, return);
+    if (m_disabledSettings.removeAll(id) > 0)
+        m_project->setNamedSettings(disabledSettingsId, m_disabledSettings);
+    if (m_enabledSettings.contains(id))
+        return;
+    m_enabledSettings << id;
+    m_project->setNamedSettings(enabledSettingsId, m_enabledSettings);
+    LanguageClientManager::applySettings(id);
+}
 
-    TextEditor::BaseTextEditor *editor = jsonEditor();
-    editor->document()->setContents(m_settings.json());
+void ProjectSettings::disableSetting(const QString &id)
+{
+    QTC_ASSERT(m_project, return);
+    if (m_enabledSettings.removeAll(id) > 0)
+        m_project->setNamedSettings(enabledSettingsId, m_enabledSettings);
+    if (m_disabledSettings.contains(id))
+        return;
+    m_disabledSettings << id;
+    m_project->setNamedSettings(disabledSettingsId, m_disabledSettings);
+    LanguageClientManager::applySettings(id);
+}
 
-    auto layout = new QVBoxLayout;
-    setLayout(layout);
-    auto group = new QGroupBox(Tr::tr("Workspace Configuration"));
-    group->setLayout(new QVBoxLayout);
-    group->layout()->addWidget(new QLabel(Tr::tr(
-        "Additional JSON configuration sent to all running language servers for this project.\n"
-        "See the documentation of the specific language server for valid settings.")));
-    group->layout()->addWidget(editor->widget());
-    layout->addWidget(group);
+void ProjectSettings::clearOverride(const QString &id)
+{
+    QTC_ASSERT(m_project, return);
+    const bool changedEnabled = m_enabledSettings.removeAll(id) > 0;
+    if (changedEnabled)
+        m_project->setNamedSettings(enabledSettingsId, m_enabledSettings);
+    const bool changedDisabled = m_disabledSettings.removeAll(id) > 0;
+    if (changedDisabled)
+        m_project->setNamedSettings(disabledSettingsId, m_disabledSettings);
+    if (changedEnabled || changedDisabled)
+        LanguageClientManager::applySettings(id);
+}
 
-    connect(editor->editorWidget()->textDocument(),
-            &TextEditor::TextDocument::contentsChanged,
-            this,
-            [=]() { m_settings.setJson(editor->document()->contents()); });
+QStringList ProjectSettings::enabledSettings()
+{
+    return m_enabledSettings;
+}
+
+QStringList ProjectSettings::disabledSettings()
+{
+    return m_disabledSettings;
+}
+
+class LanguageClientProjectSettingsWidget : public ProjectSettingsWidget
+{
+public:
+    explicit LanguageClientProjectSettingsWidget(Project *project)
+        : m_settings(project)
+    {
+        setUseGlobalSettingsCheckBoxVisible(false);
+        setGlobalSettingsId(Constants::LANGUAGECLIENT_SETTINGS_PAGE);
+        setExpanding(true);
+
+        TextEditor::BaseTextEditor *editor = createJsonEditor(this);
+        editor->document()->setContents(m_settings.json());
+
+        auto layout = new QVBoxLayout(this);
+        layout->setContentsMargins(0, 0, 0, 0);
+
+        QFormLayout *settingsLayout = nullptr;
+        for (auto settings : LanguageClientSettings::pageSettings()) {
+
+            if (settings->m_startBehavior != BaseSettings::RequiresProject)
+                continue;
+            if (!settingsLayout) {
+                auto group = new QGroupBox(Tr::tr("Project Specific Language Servers"));
+                settingsLayout = new QFormLayout;
+                settingsLayout->setFormAlignment(Qt::AlignLeft);
+                settingsLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
+                group->setLayout(settingsLayout);
+                layout->addWidget(group);
+            }
+            QComboBox *comboBox = new QComboBox;
+            comboBox->addItem(Tr::tr("Use Global Settings"));
+            comboBox->addItem(Tr::tr("Enabled"));
+            comboBox->addItem(Tr::tr("Disabled"));
+            if (m_settings.enabledSettings().contains(settings->m_id))
+                comboBox->setCurrentIndex(1);
+            else if (m_settings.disabledSettings().contains(settings->m_id))
+                comboBox->setCurrentIndex(2);
+            else
+                comboBox->setCurrentIndex(0);
+            connect(
+                comboBox,
+                &QComboBox::currentIndexChanged,
+                this,
+                [id = settings->m_id, this](int index) {
+                    if (index == 0)
+                        m_settings.clearOverride(id);
+                    else if (index == 1)
+                        m_settings.enableSetting(id);
+                    else if (index == 2)
+                        m_settings.disableSetting(id);
+                });
+            settingsLayout->addRow(settings->m_name, comboBox);
+        }
+
+        auto group = new QGroupBox(Tr::tr("Workspace Configuration"));
+        group->setLayout(new QVBoxLayout);
+        group->layout()->addWidget(new QLabel(Tr::tr(
+            "Additional JSON configuration sent to all running language servers for this project.\n"
+            "See the documentation of the specific language server for valid settings.")));
+        group->layout()->addWidget(editor->widget());
+        layout->addWidget(group);
+
+        connect(editor->editorWidget()->textDocument(), &TextEditor::TextDocument::contentsChanged,
+                this, [this, editor] { m_settings.setJson(editor->document()->contents()); });
+    }
+
+private:
+    ProjectSettings m_settings;
+};
+
+class LanguageClientProjectPanelFactory : public ProjectPanelFactory
+{
+public:
+    LanguageClientProjectPanelFactory()
+    {
+        setPriority(35);
+        setDisplayName(Tr::tr("Language Server"));
+        setId(Constants::LANGUAGECLIENT_SETTINGS_PANEL);
+        setCreateWidgetFunction([](Project *project) {
+            return new LanguageClientProjectSettingsWidget(project);
+        });
+    }
+};
+
+void setupLanguageClientProjectPanel()
+{
+    static LanguageClientProjectPanelFactory theLanguageClientProjectPanelFactory;
 }
 
 } // namespace LanguageClient
