@@ -4,9 +4,11 @@
 #include "bookmarkmanager.h"
 
 #include "bookmark.h"
+#include "texteditor.h"
 #include "texteditorconstants.h"
 #include "texteditortr.h"
 
+#include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/editormanager/editormanager.h>
@@ -15,11 +17,12 @@
 #include <coreplugin/session.h>
 
 #include <utils/algorithm.h>
+#include <utils/checkablemessagebox.h>
+#include <utils/dropsupport.h>
 #include <utils/icon.h>
 #include <utils/qtcassert.h>
-#include <utils/checkablemessagebox.h>
 #include <utils/theme/theme.h>
-#include <utils/dropsupport.h>
+#include <utils/utilsicons.h>
 
 #include <QAction>
 #include <QContextMenuEvent>
@@ -174,7 +177,7 @@ void BookmarkDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
 class BookmarkView final : public Utils::ListView
 {
 public:
-    explicit BookmarkView(BookmarkManager *manager);
+    BookmarkView();
 
     QList<QToolButton *> createToolBarWidgets();
 
@@ -189,36 +192,36 @@ protected:
     void keyPressEvent(QKeyEvent *event) final;
 
 private:
-    Core::IContext *m_bookmarkContext;
     QModelIndex m_contextMenuIndex;
-    BookmarkManager *m_manager;
 };
 
-BookmarkView::BookmarkView(BookmarkManager *manager)  :
-    m_bookmarkContext(new IContext(this)),
-    m_manager(manager)
+BookmarkView::BookmarkView()
 {
     setWindowTitle(Tr::tr("Bookmarks"));
 
-    m_bookmarkContext->setWidget(this);
-    m_bookmarkContext->setContext(Context(BOOKMARKS_CONTEXT));
+    IContext::attach(this, Context(BOOKMARKS_CONTEXT));
 
-    ICore::addContextObject(m_bookmarkContext);
-
-    ListView::setModel(manager);
+    ListView::setModel(&bookmarkManager());
 
     setItemDelegate(new BookmarkDelegate(this));
     setFrameStyle(QFrame::NoFrame);
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setAttribute(Qt::WA_MacShowFocusRect, false);
-    setSelectionModel(manager->selectionModel());
+    setSelectionModel(bookmarkManager().selectionModel());
     setSelectionMode(QAbstractItemView::SingleSelection);
     setSelectionBehavior(QAbstractItemView::SelectRows);
     setDragEnabled(true);
     setDragDropMode(QAbstractItemView::DragDrop);
 
-    connect(this, &QAbstractItemView::doubleClicked, this, &BookmarkView::gotoBookmark);
     connect(this, &QAbstractItemView::activated, this, &BookmarkView::gotoBookmark);
+    connect(this->selectionModel(), &QItemSelectionModel::currentRowChanged,
+            this, [=](const QModelIndex &current, const QModelIndex &previous) {
+        Q_UNUSED(previous)
+        Command *moveUpCmd = ActionManager::command(TextEditor::Constants::BOOKMARKS_MOVEUP_ACTION);
+        Command *moveDownCmd = ActionManager::command(TextEditor::Constants::BOOKMARKS_MOVEDOWN_ACTION);
+        moveUpCmd->action()->setEnabled(current.isValid());
+        moveDownCmd->action()->setEnabled(current.isValid());
+    });
 }
 
 QList<QToolButton *> BookmarkView::createToolBarWidgets()
@@ -238,8 +241,14 @@ QList<QToolButton *> BookmarkView::createToolBarWidgets()
 void BookmarkView::contextMenuEvent(QContextMenuEvent *event)
 {
     QMenu menu;
-    QAction *moveUp = menu.addAction(Tr::tr("Move Up"));
-    QAction *moveDown = menu.addAction(Tr::tr("Move Down"));
+    Command *moveUpCmd = ActionManager::command(TextEditor::Constants::BOOKMARKS_MOVEUP_ACTION);
+    Command *moveDownCmd = ActionManager::command(TextEditor::Constants::BOOKMARKS_MOVEDOWN_ACTION);
+    menu.addAction(moveUpCmd->action());
+    menu.addAction(moveDownCmd->action());
+    menu.addSeparator();
+    Command *sortByFilenamesCmd = ActionManager::command(TextEditor::Constants::BOOKMARKS_SORTBYFILENAMES_ACTION);
+    menu.addAction(sortByFilenamesCmd->action());
+    menu.addSeparator();
     QAction *edit = menu.addAction(Tr::tr("&Edit"));
     menu.addSeparator();
     QAction *remove = menu.addAction(Tr::tr("&Remove"));
@@ -248,20 +257,20 @@ void BookmarkView::contextMenuEvent(QContextMenuEvent *event)
 
     m_contextMenuIndex = indexAt(event->pos());
     if (!m_contextMenuIndex.isValid()) {
-        moveUp->setEnabled(false);
-        moveDown->setEnabled(false);
+        moveUpCmd->action()->setEnabled(false);
+        moveDownCmd->action()->setEnabled(false);
         remove->setEnabled(false);
         edit->setEnabled(false);
     }
 
-    if (model()->rowCount() == 0)
+    if (model()->rowCount() == 0) {
         removeAll->setEnabled(false);
+    }
 
-    connect(moveUp, &QAction::triggered, m_manager, &BookmarkManager::moveUp);
-    connect(moveDown, &QAction::triggered, m_manager, &BookmarkManager::moveDown);
+    BookmarkManager *manager = &bookmarkManager();
     connect(remove, &QAction::triggered, this, &BookmarkView::removeFromContextMenu);
     connect(removeAll, &QAction::triggered, this, &BookmarkView::removeAll);
-    connect(edit, &QAction::triggered, m_manager, &BookmarkManager::edit);
+    connect(edit, &QAction::triggered, manager, &BookmarkManager::edit);
 
     menu.exec(mapToGlobal(event->pos()));
 }
@@ -273,8 +282,8 @@ void BookmarkView::removeFromContextMenu()
 
 void BookmarkView::removeBookmark(const QModelIndex& index)
 {
-    Bookmark *bm = m_manager->bookmarkForIndex(index);
-    m_manager->deleteBookmark(bm);
+    Bookmark *bm = bookmarkManager().bookmarkForIndex(index);
+    bookmarkManager().deleteBookmark(bm);
 }
 
 void BookmarkView::keyPressEvent(QKeyEvent *event)
@@ -289,8 +298,7 @@ void BookmarkView::keyPressEvent(QKeyEvent *event)
 
 void BookmarkView::removeAll()
 {
-    if (CheckableMessageBox::question(this,
-                                      Tr::tr("Remove All Bookmarks"),
+    if (CheckableMessageBox::question(Tr::tr("Remove All Bookmarks"),
                                       Tr::tr("Are you sure you want to remove all bookmarks from "
                                              "all files in the current session?"),
                                       Key("RemoveAllBookmarks"))
@@ -298,32 +306,144 @@ void BookmarkView::removeAll()
         return;
 
     // The performance of this function could be greatly improved.
-    while (m_manager->rowCount()) {
-        QModelIndex index = m_manager->index(0, 0);
+    BookmarkManager *manager = &bookmarkManager();
+    while (manager->rowCount()) {
+        QModelIndex index = manager->index(0, 0);
         removeBookmark(index);
     }
 }
 
 void BookmarkView::gotoBookmark(const QModelIndex &index)
 {
-    Bookmark *bk = m_manager->bookmarkForIndex(index);
-    if (!m_manager->gotoBookmark(bk))
-        m_manager->deleteBookmark(bk);
+    BookmarkManager *manager = &bookmarkManager();
+    Bookmark *bk = manager->bookmarkForIndex(index);
+    if (bk && !manager->gotoBookmark(bk))
+        manager->deleteBookmark(bk);
 }
 
 ////
 // BookmarkManager
 ////
 
-BookmarkManager::BookmarkManager() :
-    m_selectionModel(new QItemSelectionModel(this, this))
+BookmarkManager::BookmarkManager(QObject *parent)
+    : QAbstractItemModel(parent), m_selectionModel(new QItemSelectionModel(this, this))
 {
+    m_editBookmarkAction.setText(Tr::tr("Edit Bookmark"));
+    m_bookmarkMarginAction.setText(Tr::tr("Toggle Bookmark"));
+
     connect(ICore::instance(), &ICore::contextChanged,
             this, &BookmarkManager::updateActionStatus);
 
     connect(SessionManager::instance(), &SessionManager::sessionLoaded,
             this, &BookmarkManager::loadBookmarks);
 
+    const Id bookmarkMenuId = "Bookmarks.Menu";
+    const Context editorManagerContext(Core::Constants::C_EDITORMANAGER);
+    const Context bookmarksContext(BOOKMARKS_CONTEXT);
+
+    MenuBuilder bookmarkMenu(bookmarkMenuId);
+    bookmarkMenu.setTitle(Tr::tr("&Bookmarks"));
+    bookmarkMenu.addToContainer(Core::Constants::M_TOOLS);
+
+    connect(&m_editBookmarkAction, &QAction::triggered, this, [this] {
+            editByFileAndLine(m_marginActionFileName, m_marginActionLineNumber);
+    });
+
+    connect(&m_bookmarkMarginAction, &QAction::triggered, this, [this] {
+            toggleBookmark(m_marginActionFileName, m_marginActionLineNumber);
+    });
+
+    ActionBuilder toggleAction(this, "Bookmarks.Toggle");
+    toggleAction.setContext(editorManagerContext);
+    toggleAction.setText(Tr::tr("Toggle Bookmark"));
+    toggleAction.setDefaultKeySequence(Tr::tr("Meta+M"), Tr::tr("Ctrl+M"));
+    toggleAction.setTouchBarIcon(Icons::MACOS_TOUCHBAR_BOOKMARK.icon());
+    toggleAction.addToContainer(bookmarkMenuId);
+    toggleAction.bindContextAction(&m_toggleAction);
+    toggleAction.addOnTriggered(this, [this] {
+        IEditor *editor = EditorManager::currentEditor();
+        auto widget = TextEditorWidget::fromEditor(editor);
+        if (widget && editor && !editor->document()->isTemporary())
+            toggleBookmark(editor->document()->filePath(), editor->currentLine());
+    });
+
+    ActionBuilder editAction(this, "Bookmarks.Edit");
+    editAction.setContext(editorManagerContext);
+    editAction.setText(Tr::tr("Edit Bookmark"));
+    editAction.setDefaultKeySequence(Tr::tr("Meta+Shift+M"), Tr::tr("Ctrl+Shift+M"));
+    editAction.addToContainer(bookmarkMenuId);
+    editAction.bindContextAction(&m_editAction);
+    editAction.addOnTriggered(this, [this] {
+        IEditor *editor = EditorManager::currentEditor();
+        auto widget = TextEditorWidget::fromEditor(editor);
+        if (widget && editor && !editor->document()->isTemporary()) {
+            const FilePath filePath = editor->document()->filePath();
+            const int line = editor->currentLine();
+            if (!hasBookmarkInPosition(filePath, line))
+                toggleBookmark(filePath, line);
+            editByFileAndLine(filePath, line);
+        }
+    });
+
+    bookmarkMenu.addSeparator();
+
+    ActionBuilder prevAction(this, Constants::BOOKMARKS_PREV_ACTION);
+    prevAction.setContext(editorManagerContext);
+    prevAction.setText(Tr::tr("Previous Bookmark"));
+    prevAction.setDefaultKeySequence(Tr::tr("Meta+,"), Tr::tr("Ctrl+,"));
+    prevAction.addToContainer(bookmarkMenuId);
+    prevAction.setIcon(Icons::PREV_TOOLBAR.icon());
+    prevAction.setIconVisibleInMenu(false);
+    prevAction.bindContextAction(&m_prevAction);
+    prevAction.addOnTriggered(this, [this] { prev(); });
+
+    ActionBuilder nextAction(this, Constants::BOOKMARKS_NEXT_ACTION);
+    nextAction.setContext(editorManagerContext);
+    nextAction.setText(Tr::tr("Next Bookmark"));
+    nextAction.setIcon(Icons::NEXT_TOOLBAR.icon());
+    nextAction.setIconVisibleInMenu(false);
+    nextAction.setDefaultKeySequence(Tr::tr("Meta+."), Tr::tr("Ctrl+."));
+    nextAction.addToContainer(bookmarkMenuId);
+    nextAction.bindContextAction(&m_nextAction);
+    nextAction.addOnTriggered(this, [this] { next(); });
+
+    bookmarkMenu.addSeparator();
+
+    ActionBuilder docPrevAction(this, "Bookmarks.PreviousDocument");
+    docPrevAction.setContext(editorManagerContext);
+    docPrevAction.setText(Tr::tr("Previous Bookmark in Document"));
+    docPrevAction.addToContainer(bookmarkMenuId);
+    docPrevAction.bindContextAction(&m_docPrevAction);
+    docPrevAction.addOnTriggered(this, [this] { prevInDocument(); });
+
+    ActionBuilder docNextAction(this, "Bookmarks.NextDocument");
+    docNextAction.setContext(Core::Constants::C_EDITORMANAGER);
+    docNextAction.setText(Tr::tr("Next Bookmark in Document"));
+    docNextAction.addToContainer(bookmarkMenuId);
+    docNextAction.bindContextAction(&m_docNextAction);
+    docNextAction.addOnTriggered(this, [this] { nextInDocument(); });
+
+    ActionContainer *touchBar = ActionManager::actionContainer(Core::Constants::TOUCH_BAR);
+    touchBar->addAction(toggleAction.command(), Core::Constants::G_TOUCHBAR_EDITOR);
+
+    ActionBuilder moveDownAction(this, Constants::BOOKMARKS_MOVEDOWN_ACTION);
+    moveDownAction.setContext(bookmarksContext);
+    moveDownAction.setText(Tr::tr("Move Down"));
+    moveDownAction.setDefaultKeySequence(Tr::tr("Ctrl+Alt+."));
+    moveDownAction.addOnTriggered(this, [this] { moveDown(); });
+
+    ActionBuilder moveUpAction(this, Constants::BOOKMARKS_MOVEUP_ACTION);
+    moveUpAction.setContext(bookmarksContext);
+    moveUpAction.setText(Tr::tr("Move Up"));
+    moveUpAction.setDefaultKeySequence(Tr::tr("Ctrl+Alt+,"));
+    moveUpAction.addOnTriggered(this, [this] { moveUp(); });
+
+    ActionBuilder sortByFilenamesAction(this, Constants::BOOKMARKS_SORTBYFILENAMES_ACTION);
+    sortByFilenamesAction.setContext(bookmarksContext);
+    sortByFilenamesAction.setText(Tr::tr("Sort by Filenames"));
+    sortByFilenamesAction.setDefaultKeySequence(Tr::tr("Ctrl+Alt+P"));
+    sortByFilenamesAction.addOnTriggered(this, [this] { sortByFilenames(); });
+
     updateActionStatus();
 }
 
@@ -528,6 +648,8 @@ void BookmarkManager::removeAllBookmarks()
 void BookmarkManager::deleteBookmark(Bookmark *bookmark)
 {
     int idx = m_bookmarksList.indexOf(bookmark);
+    if (idx < 0)
+        return;
     beginRemoveRows(QModelIndex(), idx, idx);
 
     m_bookmarksMap[bookmark->filePath()].removeAll(bookmark);
@@ -552,6 +674,7 @@ Bookmark *BookmarkManager::bookmarkForIndex(const QModelIndex &index) const
 
 bool BookmarkManager::gotoBookmark(const Bookmark *bookmark) const
 {
+    QTC_ASSERT(bookmark, return false);
     if (IEditor *editor = EditorManager::openEditorAt(
             Utils::Link(bookmark->filePath(), bookmark->lineNumber()))) {
         return editor->currentLine() == bookmark->lineNumber();
@@ -559,6 +682,16 @@ bool BookmarkManager::gotoBookmark(const Bookmark *bookmark) const
     return false;
 }
 
+void BookmarkManager::requestContextMenu(const FilePath &filePath, int lineNumber, QMenu *menu)
+{
+    m_marginActionLineNumber = lineNumber;
+    m_marginActionFileName = filePath;
+
+    menu->addAction(&m_bookmarkMarginAction);
+    if (hasBookmarkInPosition(m_marginActionFileName, m_marginActionLineNumber))
+        menu->addAction(&m_editBookmarkAction);
+}
+
 void BookmarkManager::nextInDocument()
 {
     documentPrevNext(true);
@@ -677,7 +810,15 @@ void BookmarkManager::updateActionStatus()
     IEditor *editor = EditorManager::currentEditor();
     const bool enableToggle = editor && !editor->document()->isTemporary();
 
-    emit updateActions(enableToggle, state());
+    const bool hasbm    = state() >= BookmarkManager::HasBookMarks;
+    const bool hasdocbm = state() == BookmarkManager::HasBookmarksInDocument;
+
+    m_toggleAction->setEnabled(enableToggle);
+    m_editAction->setEnabled(enableToggle);
+    m_prevAction->setEnabled(hasbm);
+    m_nextAction->setEnabled(hasbm);
+    m_docPrevAction->setEnabled(hasdocbm);
+    m_docNextAction->setEnabled(hasdocbm);
 }
 
 void BookmarkManager::move(Bookmark* mark, int newRow)
@@ -762,6 +903,7 @@ void BookmarkManager::edit()
     auto layout = new QFormLayout(&dlg);
     auto noteEdit = new QLineEdit(b->note());
     noteEdit->setMinimumWidth(300);
+    noteEdit->setFocus();
     auto lineNumberSpinbox = new QSpinBox;
     lineNumberSpinbox->setRange(1, INT_MAX);
     lineNumberSpinbox->setValue(b->lineNumber());
@@ -780,6 +922,16 @@ void BookmarkManager::edit()
     }
 }
 
+void BookmarkManager::sortByFilenames()
+{
+    beginResetModel();
+    std::sort(m_bookmarksList.begin(), m_bookmarksList.end(), [](const Bookmark* a, const Bookmark* b){
+        return a->filePath().fileName() < b->filePath().fileName()
+               || (a->filePath().fileName() == b->filePath().fileName() && a->lineNumber() < b->lineNumber());
+    });
+    endResetModel();
+}
+
 /* Returns the bookmark at the given file and line number, or 0 if no such bookmark exists. */
 Bookmark *BookmarkManager::findBookmark(const FilePath &filePath, int lineNumber)
 {
@@ -844,7 +996,7 @@ QString BookmarkManager::bookmarkToString(const Bookmark *b)
     const QLatin1Char colon(':');
     // Using \t as delimiter because any another symbol can be a part of note.
     const QLatin1Char noteDelimiter('\t');
-    return colon + b->filePath().toString() +
+    return colon + b->filePath().toUrlishString() +
             colon + QString::number(b->lineNumber()) +
             noteDelimiter + b->note();
 }
@@ -881,22 +1033,45 @@ bool BookmarkManager::isAtCurrentBookmark() const
            && currentEditor->currentLine() == bk->lineNumber();
 }
 
-// BookmarkViewFactory
+static BookmarkManager *s_bookmarkManager;
 
-BookmarkViewFactory::BookmarkViewFactory(BookmarkManager *bm)
-    : m_manager(bm)
+void setupBookmarkManager(QObject *guard)
 {
-    setDisplayName(Tr::tr("Bookmarks"));
-    setPriority(300);
-    setId("Bookmarks");
-    setActivationSequence(QKeySequence(useMacShortcuts ? Tr::tr("Alt+Meta+M") : Tr::tr("Alt+M")));
+    QTC_CHECK(!s_bookmarkManager);
+    s_bookmarkManager = new BookmarkManager(guard);
 }
 
-NavigationView BookmarkViewFactory::createWidget()
+BookmarkManager &bookmarkManager()
 {
-    auto view = new BookmarkView(m_manager);
-    view->setActivationMode(Utils::DoubleClickActivation); // QUESTION: is this useful ?
-    return {view, view->createToolBarWidgets()};
+    QTC_CHECK(s_bookmarkManager);
+    return *s_bookmarkManager;
+}
+
+// BookmarkViewFactory
+
+class BookmarkViewFactory final : public INavigationWidgetFactory
+{
+public:
+    BookmarkViewFactory()
+    {
+        setDisplayName(Tr::tr("Bookmarks"));
+        setPriority(300);
+        setId("Bookmarks");
+        setActivationSequence(QKeySequence(useMacShortcuts ? Tr::tr("Alt+Meta+M") : Tr::tr("Alt+M")));
+    }
+
+private:
+    NavigationView createWidget() final
+    {
+        auto view = new BookmarkView;
+        view->setActivationMode(Utils::DoubleClickActivation); // QUESTION: is this useful ?
+        return {view, view->createToolBarWidgets()};
+    }
+};
+
+void setupBookmarkView()
+{
+    static BookmarkViewFactory theBookmarkViewFactory;
 }
 
 } // TextEditor::Internal
